package dev.pcvolkmer.onco.datamapper.mapper;

import dev.pcvolkmer.mv64e.mtb.*;
import dev.pcvolkmer.onco.datamapper.PropertyCatalogue;
import dev.pcvolkmer.onco.datamapper.ResultSet;
import dev.pcvolkmer.onco.datamapper.datacatalogues.EinzelempfehlungCatalogue;
import dev.pcvolkmer.onco.datamapper.datacatalogues.TherapieplanCatalogue;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import java.util.List;
import java.util.Map;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.anyInt;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.doAnswer;

@ExtendWith(MockitoExtension.class)
class TherapieplanDataMapperTest {

    TherapieplanCatalogue therapieplanCatalogue;
    EinzelempfehlungCatalogue einzelempfehlungCatalogue;
    PropertyCatalogue propertyCatalogue;

    TherapieplanDataMapper dataMapper;

    @BeforeEach
    void setUp(
            @Mock TherapieplanCatalogue therapieplanCatalogue,
            @Mock EinzelempfehlungCatalogue einzelempfehlungCatalogue,
            @Mock PropertyCatalogue propertyCatalogue
    ) {
        this.therapieplanCatalogue = therapieplanCatalogue;
        this.einzelempfehlungCatalogue = einzelempfehlungCatalogue;
        this.propertyCatalogue = propertyCatalogue;
        this.dataMapper = new TherapieplanDataMapper(therapieplanCatalogue, einzelempfehlungCatalogue, propertyCatalogue);
    }

    @Test
    void shouldCreateCarePlan(@Mock ResultSet resultSet) {
        final var testData = Map.of(
                "id", "1",
                "patient_id", "42",
                "wirkstoffe_json", "[{\"code\":\"\",\"name\":\"PARP-Inhibierung\",\"system\":\"UNREGISTERED\"}]",
                "protokollauszug", "Das ist ein Protokollauszug"
        );

        doAnswer(invocationOnMock -> {
            var columnName = invocationOnMock.getArgument(0, String.class);
            return testData.get(columnName);
        }).when(resultSet).getString(anyString());

        doAnswer(invocationOnMock -> resultSet)
                .when(therapieplanCatalogue)
                .getById(anyInt());

        doAnswer(invocationOnMock -> List.of(resultSet))
                .when(einzelempfehlungCatalogue)
                .getAllByParentId(anyInt());

        var actual = this.dataMapper.getById(1);
        assertThat(actual).isInstanceOf(MtbCarePlan.class);
        assertThat(actual.getId()).isEqualTo("1");
        assertThat(actual.getPatient())
                .isEqualTo(Reference.builder().id("42").type("Patient").build());

        assertThat(actual.getMedicationRecommendations()).hasSize(1);

        assertThat(actual.getNotes()).hasSize(1);
        assertThat(actual.getNotes().get(0)).isEqualTo("Das ist ein Protokollauszug");
    }

    @Test
    void shouldSetRecommendationsMissingReason() {
        final Map<String, Object> testData = Map.of(
                "id", 1,
                "patient_id", 42,
                "status_begruendung", "no-target"
        );

        doAnswer(invocationOnMock -> ResultSet.from(testData))
                .when(therapieplanCatalogue)
                .getById(anyInt());

        var actual = this.dataMapper.getById(1);
        assertThat(actual).isInstanceOf(MtbCarePlan.class);
        assertThat(actual.getId()).isEqualTo("1");

        assertThat(actual.getRecommendationsMissingReason())
                .isEqualTo(
                        MtbCarePlanRecommendationsMissingReasonCoding.builder()
                                .code(MtbCarePlanRecommendationsMissingReasonCodingCode.NO_TARGET)
                                .build()
                );

        assertThat(actual.getNoSequencingPerformedReason()).isNull();
    }

    @Test
    void shouldSetNoSequencingPerformedReason() {
        final Map<String, Object> testData = Map.of(
                "id", 1,
                "patient_id", 42,
                "status_begruendung", "non-genetic-cause"
        );

        doAnswer(invocationOnMock -> ResultSet.from(testData))
                .when(therapieplanCatalogue)
                .getById(anyInt());

        var actual = this.dataMapper.getById(1);
        assertThat(actual).isInstanceOf(MtbCarePlan.class);
        assertThat(actual.getId()).isEqualTo("1");

        assertThat(actual.getRecommendationsMissingReason()).isNull();


        assertThat(actual.getNoSequencingPerformedReason()).isEqualTo(
                CarePlanNoSequencingPerformedReasonCoding.builder()
                        .code(NoSequencingPerformedReasonCode.NON_GENETIC_CAUSE)
                        .build()
        );
    }

}
